@@ -521,6 +521,19 @@ def _start(client, bot):
 					line=line.replace("$emojiUrl"+bot._style[0]+line.split("$emojiUrl"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],str(emoji.url))
 				except Exception as e:
 					line=line.replace("$emojiUrl"+bot._style[0]+line.split("$emojiUrl"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
+		while "$isNumber"+bot._style[0] in line:
+			if nowloop==line:
+				break
+			else:
+				nowloop=line
+				try:
+					try:
+						int(line.split("$isNumber"+bot._style[0])[-1].split(bot._style[1])[0])
+						line=line.replace("$isNumber"+bot._style[0]+line.split("$isNumber"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"true")
+					except:
+						line=line.replace("$isNumber"+bot._style[0]+line.split("$isNumber"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"false")
+				except Exception as e:
+					line=line.replace("$isNumber"+bot._style[0]+line.split("$isNumber"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
 		line=get_vars(line, guild, vars, author)
 		while "$replaceText"+bot._style[0] in line:
 			if nowloop==line:
@@ -604,6 +617,44 @@ def _start(client, bot):
 						line=line.replace("$checkCondition"+bot._style[0]+line.split("$checkCondition"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"false")
 				except Exception as e:
 					line=line.replace("$checkCondition"+bot._style[0]+line.split("$checkCondition"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], "none")
+		while "$leaderboard"+bot._style[0] in line:
+			if nowloop==line:
+				break
+			else:
+				nowloop=line
+				try:
+					top={}
+					topos=1
+					topstr=""
+					if line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]=="user":
+						data=bot._db.data[line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]][line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[1].split("&")[1]]
+					else:
+						data=bot._db.data[line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]]
+					for u in data:
+						try:
+							top[str(u)]=data[u][line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[1].split("&")[0]]
+							topos+=1
+						except:
+							pass
+					top=sorted(top.items(), key=lambda x: x[1], reverse=True)
+					topos=0
+					for t, u in top:
+						try:
+							if line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]=="server":
+								t = discord.utils.get(client.guilds, id=int(t))
+							else:
+								t = discord.utils.get(client.get_all_members(), id=int(t))
+							if t:
+								topos+=1
+								topstr+=str(topos)+". "+str(t)+" - "+str(u)+"\n"
+						except:
+							pass
+					while len(topstr)>2000:
+						topstr="\n".join(topstr.split("\n")[:-2])
+					line=line.replace("$leaderboard"+bot._style[0]+line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], str(topstr))
+				except Exception as e:
+					print(e)
+					line=line.replace("$leaderboard"+bot._style[0]+line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], "none")
 		return line
 	
 	async def use(message, name, trigger, type):
@@ -643,9 +694,11 @@ def _start(client, bot):
 				lines.remove("")
 			for line in lines:
 				beforeline=line
+				line
 				if go==0:
 					line=line.lstrip(" ")
 					try:
+						line=line.replace("/n","\n")
 						if line.startswith("$cooldown"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$cooldown"+bot._style[0]):]
@@ -686,6 +739,68 @@ def _start(client, bot):
 								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								mes=await ctx.fetch_message(int(line))
 								await mes.delete()
+						elif line.startswith("$editRole"+bot._style[0]):
+							if line[-1]==bot._style[1]:
+								line=line[len("$editRole"+bot._style[0]):]
+								line=line[:-1]
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
+								r=discord.utils.get(guild.roles,id=int(line.split(bot._separator)[0]))
+								l=l[len(line.split(bot._separator)[0]+bot._separator):]
+								for l in line.split(bot._separator):
+									if l.startswith("name="):
+										l=l[len("name="):]
+										await r.edit(name=l)
+									elif l.startswith("color="):
+										l=l[len("color="):]
+										rgb=list(int(l[i:i+2], 16) for i in (0, 2, 4))
+										await r.edit(color=discord.Colour.from_rgb(rgb[0],rgb[1],rgb[2]))
+									elif l.startswith("pos="):
+										l=l[len("pos="):]
+										await r.edit(position=int(l))
+									elif l.startswith("perms="):
+										l=l[len("perms="):].split("&")
+										permissions = r.permissions
+										if l[0]=="kick":
+											permissions.update(kick_members = eval(l[1]))
+										elif l[0]=="ban":
+											permissions.update(ban_members = eval(l[1]))
+										elif l[0]=="nick":
+											permissions.update(change_nickname = eval(l[1]))
+										elif l[0]=="admin":
+											permissions.update(administrator = eval(l[1]))
+										await r.edit(permissions=permissions)
+						elif line.startswith("$rainbowRole"+bot._style[0]):
+							if line[-1]==bot._style[1]:
+								line=line[len("$rainbowRole"+bot._style[0]):]
+								line=line[:-1]
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
+								colours = [discord.Color.dark_orange(),discord.Color.orange(),discord.Color.dark_gold(),discord.Color.gold(),discord.Color.dark_magenta(),discord.Color.magenta(),discord.Color.red(),discord.Color.dark_red(),discord.Color.blue(),discord.Color.dark_blue(),discord.Color.teal(),discord.Color.dark_teal(),discord.Color.green(),discord.Color.dark_green(),discord.Color.purple(),discord.Color.dark_purple()]
+								r=discord.utils.get(guild.roles,id=int(line.split(bot._separator)[1]))
+								rgb=list(int(str(r.color).replace("#","")[i:i+2], 16) for i in (0, 2, 4))
+								st=discord.Colour.from_rgb(rgb[0],rgb[1],rgb[2])
+								t=0
+								line=line.split(bot._separator)[0]
+								if line.endswith("s"):
+									line=line.rstrip("s")
+								elif line.endswith("m"):
+									line=line.rstrip("m")
+									line=int(line)*60
+								elif line.endswith("h"):
+									line=line.rstrip("h")
+									line=int(line)*60*60
+								elif line.endswith("d"):
+									line=line.rstrip("d")
+									line=int(line)*60*60*24
+								else:
+									line=int(line)
+								while t<=int(line):
+									try:
+										await r.edit(color=random.choice(colours))
+									except Exception:
+										pass
+									t+=0.5
+									await asyncio.sleep(0.5)
+								await r.edit(color=st)
 						elif line.startswith("$serverCooldown"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$serverCooldown"+bot._style[0]):]
@@ -1140,7 +1255,7 @@ def _start(client, bot):
 								if n:
 									t=not t
 								if t==False:
-										go+=int(cd)
+									go+=int(cd)
 						elif line.startswith("$clear"+bot._style[0]):
 							line=line[len("$clear"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
